// pgp4pine main.c

// See common.c for the version and license.
// Copyright (C) 1998 by Chris Wiegand

#include "includes.h"

// ----------------------------------------------------------------

int main(int argc,char **argv)
{
	char recipients[EMAIL_ADDRESS_MAX_LENGTH];
	char recipientsTmp[EMAIL_ADDRESS_MAX_LENGTH];

	int showWelcome=1;
	int i, j, pipeFiles = -1, keepTmpFile = -1;
	int outgoing=0,myClearScreen=1;

	char inFile[MAX_COMMAND_LINE_LENGTH];
	char outFile[MAX_COMMAND_LINE_LENGTH];

	char mainTmpFile[MAX_COMMAND_LINE_LENGTH];

	int signFile=0,encryptFile=0,armorFile=0,canonFile=0,detachedSig=0; 
		// set in command line...

	openDebugFile(); // ABSOLUTELY MUST BE FIRST!!!!!

	secretKeyring = NULL;
	publicKeyring = NULL;

	if (argc == 1) {
		printUsage();
		exit(-1);
        }
	if (strcmp(argv[1],"--version") == 0) {
		printVersion();
		exit(-1);
	}
	if (strcmp(argv[1],"--usage") == 0) {
		printUsage();
		exit(-1);
	}

	// Here we initialize the recipients list
	strcpy(recipients,""); // initialize it...
	strcpy(recipientsTmp,""); // initialize it too...
	strcpy(mainTmpFile,""); // clear this out (Dale Osowski found this one..)

	sprintf(debugLine,"main: parsing arguments 1..%d\n",argc-1);
	documentStatus(debugLine);
	
	for (i=1; i<=argc-1; i++) {
		sprintf(debugLine,"main: parsing argument %d = -%s-\n",i,argv[i]);
		documentStatus(debugLine);

		// This stuff can be anywhere...
		if (strcmp(argv[i],"-e") == 0 || strcmp(argv[i],"--encrypt") == 0) outgoing = 1;
		if (strcmp(argv[i],"-s") == 0 || strcmp(argv[i],"--sign") == 0) outgoing = 1;
		if (strcmp(argv[i],"-d") == 0 || strcmp(argv[i],"--decrypt") == 0) outgoing = -1;
		if (strcmp(argv[i],"-sf+") == 0 || strcmp(argv[i],"-sf") == 0) signFile = 1;
		if (strcmp(argv[i],"-sf-") == 0) signFile = -1;
		if (strcmp(argv[i],"-ef+") == 0 || strcmp(argv[i],"-ef") == 0) encryptFile = 1;
		if (strcmp(argv[i],"-ef-") == 0) encryptFile = -1;
		if (strcmp(argv[i],"-af+") == 0 || strcmp(argv[i],"-af") == 0) armorFile = 1;
		if (strcmp(argv[i],"-af-") == 0) armorFile = -1;
		if (strcmp(argv[i],"-uf+") == 0 || strcmp(argv[i],"-uf") == 0) canonFile = 1;
		if (strcmp(argv[i],"-uf-") == 0) canonFile = -1;
		if (strcmp(argv[i],"-ds+") == 0 || strcmp(argv[i],"-ds") == 0) detachedSig = 1;
		if (strcmp(argv[i],"-ds-") == 0) detachedSig = -1;
		if (strcmp(argv[i],"--noclear") == 0) myClearScreen = -1;
		if (strcmp(argv[i],"-k") == 0 || strcmp(argv[i],"--keep") == 0) keepTmpFile = 1;
		if (strcmp(argv[i],"-f") == 0 || strcmp(argv[i],"--pipe") == 0) pipeFiles = 1;
		if (strcmp(argv[i],"-q") == 0 || strcmp(argv[i],"--quiet") == 0) showWelcome = -1;
		if (i<argc-1) {
			// stuff that reads the NEXT argv[] goes inside here...
			if (strcmp(argv[i],"-t") == 0 || strcmp(argv[i],"--tempfile") == 0) strcpy(mainTmpFile,argv[i+1]);
			if (strcmp(argv[i],"-i") == 0 || strcmp(argv[i],"--infile") == 0) strcpy(inFile,argv[i+1]);
			if (strcmp(argv[i],"-o") == 0 || strcmp(argv[i],"--outfile") == 0) strcpy(outFile,argv[i+1]);
			if (strcmp(argv[i],"-r") == 0 || strcmp(argv[i],"--recipients") == 0) {
				strcpy(recipientsTmp," "); // to make it blank...
				for (j=i+1;j<=argc-1;j++) {
					sprintf(debugLine,"main: subparsing recipient %d = -%s-\n",j,argv[j]);
					documentStatus(debugLine);
					strcat(recipientsTmp,argv[j]);
					strcat(recipientsTmp," ");
				}
				strcpy(recipients,recipientsTmp);
				i=argc; // so that we end...
				sprintf(debugLine,"main: recipients: %s\n",recipients);
				documentStatus(debugLine);
			}
		}
	}

#ifndef MY_DEBUG	
	if (myClearScreen > 0) clearScreen();
#endif

	if (showWelcome > 0) printVersion();  // we introduce ourselves...
	readRCFile();    // read our prefs...
	selectProfile(); // and get the profile for this session...

	// now we want to merge the profile we selected with the options from the command line...
	if (signFile != 0) myUserPrefs->autoSign = signFile;
	if (encryptFile != 0) myUserPrefs->autoEncrypt = encryptFile;
	if (armorFile != 0) myUserPrefs->asciiArmor = armorFile;
	if (canonFile != 0) myUserPrefs->universalText = canonFile;
	if (detachedSig != 0) myUserPrefs->detachedSig = detachedSig;
	
	if (strlen(inFile) < 3) {
		if (pipeFiles > 0) {
			// If we're piping, we use /tmp/pgp4pine.tmp unless
			// it's overridden in the command line.
			// If we're not piping, we set that in the command line anyways
			strcpy(inFile,"/tmp/pgp4pine.tmp");
		} else {
			// we're not piping...
			printf("You're not piping, yet no input file was found in the command line.\nUse -i (fileName) to specify one.\n");
			exit(-1);
		}
	}

	// If we don't set an outfile, we use the infile...
	if (strlen(outFile) < 3) strcpy(outFile,inFile);

	if (outgoing == 0) {
		printf("You must specify either -e to sign/encrypt or -d to verify/decrypt.\n");
		exit(-1);
	}

	if (pipeFiles > 0) pipeSTDINToFile(inFile);

	if (outgoing == 1) {
		// just before we encrypt, get key rings and check recipients!
		getSecretKeyring();
		getPublicKeyring();
		chooseSecretKey();
		// sending the file/message...
		if (strlen(mainTmpFile) > 1) {
			remove(mainTmpFile);
			fileSignEncryptMenu(inFile,mainTmpFile,recipients);
		} else if (strlen(myUserPrefs->myTmpFile) > 1) {
			remove(myUserPrefs->myTmpFile);
			fileSignEncryptMenu(inFile,myUserPrefs->myTmpFile,recipients);
		} else {
			fileSignEncryptMenu(inFile,outFile,recipients);
		}
	} else {
		// receiving the file/message
		if (strlen(mainTmpFile) > 1) {
			remove(mainTmpFile);
			fileVerifyDecryptMenu(inFile,mainTmpFile);
		} else if (strlen(myUserPrefs->myTmpFile) > 1) {
			remove(myUserPrefs->myTmpFile);
			fileVerifyDecryptMenu(inFile,myUserPrefs->myTmpFile);
		} else {
			fileVerifyDecryptMenu(inFile,outFile);
		}
	}

	if (strlen(mainTmpFile) > 1) {
		char mainTmpCommand[MAX_COMMAND_LINE_LENGTH];
		sprintf(mainTmpCommand,"cp -f %s %s",mainTmpFile,outFile);
		runProcessFG(mainTmpCommand);
		if (keepTmpFile < 0) remove(mainTmpFile);
	} else {
		if (strlen(myUserPrefs->myTmpFile) > 1) {
			char mainTmpCommand[MAX_COMMAND_LINE_LENGTH];
			sprintf(mainTmpCommand,"cp -f %s %s",myUserPrefs->myTmpFile,outFile);
			runProcessFG(mainTmpCommand);
			if (keepTmpFile < 0) remove(myUserPrefs->myTmpFile);
		}
	}
	
	if (pipeFiles > 0) {
		char tmpAppString[MAX_COMMAND_LINE_LENGTH];
		sprintf(tmpAppString,"%s %s",prefsMyPager,outFile);
		runProcessFG(tmpAppString);
		if (keepTmpFile < 0) remove(outFile);
		if (keepTmpFile < 0 && strcmp(inFile,outFile) != 0) remove(inFile);
	}
	
	// dealloc secret and public keyrings!
	documentStatus("Deallocating SECRET Keyring...");
	if (secretKeyring != NULL) deallocAllKeys(secretKeyring);
	secretKeyring = NULL;

	documentStatus("Deallocating PUBLIC keyring...");
	if (publicKeyring != NULL) deallocAllKeys(publicKeyring);
	publicKeyring = NULL;

	documentStatus("almost closed...\n");

#ifdef MY_DEBUG
	fflush(debugFile);
	fclose(debugFile);
#endif
	return 0;
}

