// pgp4pine profiles.c

// See common.c for the version and license.
// Copyright (C) 1998 by Chris Wiegand

#include "includes.h"

// -----------------------------------------------------------------

void clearProfiles(struct myProfile *firstProfile)
{
	struct myProfile *nextProfile2 = NULL;
	do {
		nextProfile2 = firstProfile->nextProfile;
		myFree(firstProfile);
		firstProfile = nextProfile2;
	} while (nextProfile2 != NULL);
}

void selectProfile()
{
	struct myProfile *firstProfile = NULL;
	struct myProfile *thisProfile = NULL;
	struct myProfile *prevProfile = NULL;
	char profileList[PREFS_LINE_LENGTH];
	char tmpString[PREFS_LINE_LENGTH];
	int p=0, sel=0, profileCount = 0;
	char thisProfileName[PREFS_LINE_LENGTH];

	printf("\nLoading profiles"); fflush(stdout);
	strcpy(profileList,readPrefs("profile_list"," "));
	if (strlen(profileList)<2) {
		sprintf(profileList,"default");
		printf("\nWarning: No profiles were defined, so I'm using a default profile.\nThis means I'm using PGP 5, nothing automatic, asking everything.\nTo find out more about profiles, type man pgp4pine.\n");
		printf("Loading profiles");
	}
	strcat(profileList,":.:"); // this seems to make it work...

	strcpy(thisProfileName,strtok(profileList,":\n"));
	do {
		printf(" %s ",thisProfileName); fflush(stdout);
		thisProfile = (struct myProfile *) myMalloc(sizeof(myProfileStruct));
		thisProfile->nextProfile = NULL;
		if (firstProfile == NULL) firstProfile = thisProfile;
		if (prevProfile != NULL) prevProfile->nextProfile = thisProfile;
		prevProfile = thisProfile;
		strcpy(thisProfile->profileName,thisProfileName);
		strcpy(thisProfileName,strtok('\0',":\n"));
		stillAlive();
		profileCount++;
	} while (strlen(thisProfileName) > 2);

	printf(" ");
	thisProfile = firstProfile;
	// now that we've allocated enough memory, let's load the actual items...	
	do {
		strcpy(thisProfileName,thisProfile->profileName);
		
		sprintf(tmpString,"profile_%s_version",thisProfileName);
		thisProfile->pgpVersion = atoi(readPrefs(tmpString,"5"));

		sprintf(tmpString,"profile_%s_folder",thisProfileName);
		strcpy(thisProfile->pgpFolder,readPrefs(tmpString,""));

		sprintf(tmpString,"profile_%s_extra_options",thisProfileName);
		strcpy(thisProfile->extraOptions,readPrefs(tmpString," "));

		sprintf(tmpString,"profile_%s_autosign",thisProfileName);
		thisProfile->autoSign = atoi(readPrefs(tmpString,"0"));

		sprintf(tmpString,"profile_%s_autoencrypt",thisProfileName);
		thisProfile->autoEncrypt = atoi(readPrefs(tmpString,"0"));

		sprintf(tmpString,"profile_%s_encrypt_to_self",thisProfileName);
		thisProfile->encryptToSelf = atoi(readPrefs(tmpString,"1"));

		sprintf(tmpString,"profile_%s_add_new_keys",thisProfileName);
		thisProfile->addNewKeys = atoi(readPrefs(tmpString,"1"));

		sprintf(tmpString,"profile_%s_my_address",thisProfileName);
		strcpy(thisProfile->myAddress,readPrefs(tmpString," "));
		
		sprintf(tmpString,"profile_%s_ascii_armor",thisProfileName);
		thisProfile->asciiArmor = atoi(readPrefs(tmpString,"1"));
		
		sprintf(tmpString,"profile_%s_universal_text",thisProfileName);
		thisProfile->universalText = atoi(readPrefs(tmpString,"1"));

		sprintf(tmpString,"profile_%s_tmpfile",thisProfileName);
		strcpy(thisProfile->myTmpFile,readPrefs(tmpString," "));
		
		sprintf(tmpString,"profile_%s_detached_sigs",thisProfileName);
		thisProfile->detachedSig = atoi(readPrefs(tmpString,"-1"));

		thisProfile = thisProfile->nextProfile;
		stillAlive();
	} while (thisProfile != NULL);

	printf("\n");	
	if (profileCount > 1) {
		// okay, now we have the profiles loaded, let's choose...
		printf("\nPlease select a profile below.\n");
		thisProfile = firstProfile;
		p = 1;
		do {
			switch (thisProfile->pgpVersion) {
				case 1:
					printf("%i: GnuPG Profile \"%s\"\n",p,thisProfile->profileName); break;
				case 2:
					printf("%i: PGP 2 Profile \"%s\"\n",p,thisProfile->profileName); break;
				case 5:
					printf("%i: PGP 5 Profile \"%s\"\n",p,thisProfile->profileName); break;
			}
			thisProfile = thisProfile->nextProfile;
			p++;
		} while (thisProfile != NULL);
		p--;
		sel = ('a'-askAlphaRange("Please enter a valid selection: ",'a',('a'+p-1))+1);
		printf("-%d-",sel);
	} else {
		sel = 1; // we want the first choice, the only one...
	}
	
	thisProfile = firstProfile; // set back to beginning...
	for (p=1;p<sel;p++) thisProfile = thisProfile->nextProfile; // skip to profile we want

	// now that we have the profile, let's set the right global variables...
	myUserPrefs = (struct myProfile *) myMalloc(sizeof(myProfileStruct));
	strcpy(myUserPrefs->profileName,thisProfile->profileName);
	myUserPrefs->pgpVersion = thisProfile->pgpVersion;
	strcpy(myUserPrefs->pgpFolder,thisProfile->pgpFolder);
	strcpy(myUserPrefs->extraOptions,thisProfile->extraOptions);
	myUserPrefs->autoSign = thisProfile->autoSign;
	myUserPrefs->autoEncrypt = thisProfile->autoEncrypt;
	myUserPrefs->encryptToSelf = thisProfile->encryptToSelf;
	myUserPrefs->addNewKeys = thisProfile->addNewKeys;
	strcpy(myUserPrefs->myAddress,thisProfile->myAddress);
	myUserPrefs->asciiArmor = thisProfile->asciiArmor;
	myUserPrefs->universalText = thisProfile->universalText;
	strcpy(myUserPrefs->myTmpFile,thisProfile->myTmpFile);
	myUserPrefs->detachedSig = thisProfile->detachedSig;

	clearProfiles(firstProfile); // dealloc...
	printf("\n");
}

